@@ -3,14 +3,8 @@
 class Api::V1::Timelines::BaseController < Api::BaseController
   after_action :insert_pagination_headers, unless: -> { @statuses.empty? }
 
-  before_action :require_user!, if: :require_auth?
-
   private
 
-  def require_auth?
-    !Setting.timeline_preview
-  end
-
   def pagination_collection
     @statuses
   end
