@@ -13,8 +13,12 @@ RSpec.describe PublishScheduledStatusWorker do
     end
 
     context 'when the account is not disabled' do
+      let(:user) { Fabricate(:user) }
+      let(:scheduled_status) { Fabricate(:scheduled_status, account: user.account, params: { text: 'Hello world, future!', quoted_status_id: Fabricate(:status, account: user.account).id }) }
+
       it 'creates a status and removes scheduled record' do
         expect(scheduled_status.account.statuses.first.text).to eq 'Hello world, future!'
+        expect(scheduled_status.account.statuses.first.quote).to_not be_nil
 
         expect(ScheduledStatus.find_by(id: scheduled_status.id)).to be_nil
       end
